@@ -1188,8 +1188,10 @@ void copycat()
       for (ops = oldobj->params; ops != NULL; ops = ops->next) {
 	 newops = (oparamptr)malloc(sizeof(oparam));
 	 newops->next = (*newobj)->params;
+	 newops->key = strdup(ops->key);
 	 (*newobj)->params = newops;
 	 newops->type = ops->type;
+	 newops->which = ops->which;
 	 switch (ops->type) {
 	    case XC_INT:
 	       newops->parameter.ivalue = ops->parameter.ivalue;
@@ -1287,15 +1289,7 @@ void copycat()
 
 	       clabel = TOLABEL(oldobj->plist + i);
 	       NEW_LABEL(nlabel, (*newobj));
-	       (*nlabel)->rotation = clabel->rotation;
-	       (*nlabel)->color = clabel->color;
-	       (*nlabel)->position.x = clabel->position.x;
-	       (*nlabel)->position.y = clabel->position.y;
-	       (*nlabel)->scale = clabel->scale;
-	       (*nlabel)->justify = clabel->justify;
-	       (*nlabel)->string = stringcopy(clabel->string);
-	       (*nlabel)->passed = NULL;
-	       (*nlabel)->pin = clabel->pin;
+	       labelcopy(*nlabel, clabel);
                } break;
       
 	    case(OBJINST): {
@@ -1303,25 +1297,7 @@ void copycat()
 
 	       cinst = TOOBJINST(oldobj->plist + i);
 	       NEW_OBJINST(ninst, (*newobj));
-	       (*ninst)->rotation = cinst->rotation;
-	       (*ninst)->color = cinst->color;
-	       (*ninst)->position.x = cinst->position.x;
-	       (*ninst)->position.y = cinst->position.y;
-	       (*ninst)->scale = cinst->scale;
-	       (*ninst)->thisobject = cinst->thisobject;
-	       (*ninst)->passed = NULL;
-
-	       /* Library instance's parameter list is always default (null) */
-	       (*ninst)->params = NULL;
-	       (*ninst)->bbox.lowerleft.x = cinst->thisobject->bbox.lowerleft.x;
-	       (*ninst)->bbox.lowerleft.y = cinst->thisobject->bbox.lowerleft.y;
-	       (*ninst)->bbox.width = cinst->thisobject->bbox.width;
-	       (*ninst)->bbox.height = cinst->thisobject->bbox.height;
-
-	       (*ninst)->schembbox = NULL;
-
-	       /* here---deal with attached schematics */
-
+	       instcopy(*ninst, cinst);
                } break;
          }
 	 (*newobj)->parts++;
