@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <math.h>
 
 #include <X11/Intrinsic.h>
@@ -80,7 +75,7 @@ Boolean checkselect(short value)
  
    value &= areastruct.filter;	/* apply the selection filter */
 
-   if (areastruct.selects == 0) objectselect(value);
+   if (areastruct.selects == 0) select_element(value);
    if (areastruct.selects == 0) return False;
    for (check = areastruct.selectlist; check < areastruct.selectlist +
 	areastruct.selects; check++)
@@ -89,21 +84,17 @@ Boolean checkselect(short value)
    else return True;
 }
 
-/*-----------------------------------------------------------*/
-/* Select list numbering revision for routine objectdelete() */
-/*-----------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Select list numbering revision when an element is deleted	*/
+/* from an object. 						*/
+/*--------------------------------------------------------------*/
 
-void reviseselect(short *selectobj)
+void reviseselect(short *slist, int selects, short *removed)
 {
    short *chkselect;
 
-   for (chkselect = selectobj + 1; chkselect < areastruct.selectlist
-	  + areastruct.selects; chkselect++)
-      if (*chkselect > *selectobj) (*chkselect)--;
-
-   for (chkselect = areastruct.selectlast; chkselect < areastruct.selectlast
-	  + areastruct.lastselects; chkselect++)
-      if (*chkselect > *selectobj) (*chkselect)--;
+   for (chkselect = slist; chkselect < slist + selects; chkselect++)
+      if (*chkselect > *removed) (*chkselect)--;
 }
 
 /*----------------------*/
@@ -114,7 +105,7 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
 {
    genericptr elementptr = *(curobj->plist + instance);
 
-   switch ((*(curobj->plist + instance))->type) {
+   switch (ELEMENTTYPE(*(curobj->plist + instance))) {
       case ARC:
          UDrawArc((arcptr)elementptr);
 	 break;
@@ -130,7 +121,7 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
       case LABEL:
          UDrawString((labelptr)elementptr, mode, curinst);
 	 break;
-      case OBJECT:
+      case OBJINST:
          UDrawObject((objinstptr)elementptr, SINGLE, mode, NULL);
          break;
    }
@@ -151,7 +142,6 @@ void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
    XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-/* XSetForeground(dpy, areastruct.gc, PARAMCOLOR ^ BACKGROUND); */
    XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
 }
@@ -345,7 +335,7 @@ Boolean pathselect(genericptr *curgen, short class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
+short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr selinst)
 {
    short	*newselect, *foundtmp, *foundlist;
 
@@ -380,7 +370,7 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
 
       /* check for position inside the bounding box of one of the objects */
 
-      if ((*curgen)->type == (class & OBJECT)) {
+      if ((*curgen)->type == (class & OBJINST)) {
 
 	 objinstbbox(TOOBJINST(curgen), newboxpts);
 
@@ -401,10 +391,8 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
 
 	 labelptr curlab = TOLABEL(curgen);
 
-#ifdef SCHEMA
 	 /* Don't select temporary labels from schematic capture system */
 	 if (curlab->string->type != FONT_NAME) continue;
-#endif
 
 	 labelbbox(curlab, newboxpts, selinst);
 
@@ -573,6 +561,11 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
    /* snap the coordinates */
 
    u2u_snap(&areastruct.save);
+
+   /* Register the selection as an undo event */
+   register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+
    return newselect;
 }
 
@@ -585,7 +578,7 @@ Boolean areaelement(genericptr *curgen)
    Boolean selected;
    pointlist    currentpt;
 
-   switch((*curgen)->type) {
+   switch(ELEMENTTYPE(*curgen)) {
 
       case(ARC):
 	   /* check center of arcs */
@@ -661,8 +654,8 @@ void selectarea()
       /* apply the selection filter */
       if (!((*curgen)->type & areastruct.filter)) continue;
 
-      switch((*curgen)->type) {
-	case(OBJECT):
+      switch(ELEMENTTYPE(*curgen)) {
+	case(OBJINST):
            /* check for instance of object inside area box */
            selected = (TOOBJINST(curgen)->position.x < areastruct.save.x) &&
 		(TOOBJINST(curgen)->position.x > areastruct.origin.x) &&
@@ -678,10 +671,8 @@ void selectarea()
 
 	   selected = False;
 
-#ifdef SCHEMA
 	   /* Ignore temporary labels created by the netlist generator */
 	   if (slab->string->type != FONT_NAME) break;
-#endif
 
 	   /* create a 4-point polygon out of the select box information */
 	   bboxpts[0].x = bboxpts[3].x = areastruct.origin.x;
@@ -693,14 +684,13 @@ void selectarea()
 	   InvTransformPoints(bboxpts, newboxpts, 4, slab->position,
 		  slab->scale, slab->rotation);
 
-#ifdef SCHEMA
 	   if (slab->pin) {
 	      if (!areastruct.schemon) continue;
 	      for (j = 0; j < 4; j++) 
 		 pinadjust(slab->justify, &(newboxpts[j].x),
 			&(newboxpts[j].y), -1);
 	   }
-#endif
+
 	   tmpext = ULength(slab->string, areastruct.topinstance, 0.0, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
 			tmpext.width : tmpext.width >> 1) : 0);
@@ -782,6 +772,10 @@ void selectarea()
       textpos = textend = 0;
    }
 
+   /* Register the selection as an undo event */
+   register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+
    /* Drawing of selected objects will take place when drawarea() is */
    /* executed after the button release.			     */
 }
@@ -796,7 +790,7 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
       if (areastruct.selects == 0)
 	 Wprintf("Nothing to deselect!");
       else if (areastruct.selects == 1)
-	 objectdeselect();
+	 unselect_all();
       else {
          eventmode = DESEL_MODE;
          Wprintf("Click on element to deselect");
@@ -804,11 +798,23 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
    }
 }
 
+/*------------------------------------------------------*/
+/* Redraw all the selected objects in the select color.	*/
+/*------------------------------------------------------*/
+
+void draw_all_selected()
+{
+   int j;
+
+   for (j = 0; j < areastruct.selects; j++)
+      gendrawselected(areastruct.selectlist + j, topobject, areastruct.topinstance);
+}
+
 /*---------------------------------------------------------*/
 /* Redraw all the selected objects in their normal colors. */
 /*---------------------------------------------------------*/
 
-void drawselects(objectptr thisobj, objinstptr thisinst)
+void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
    short *lastselect;
 
@@ -836,13 +842,9 @@ void drawselects(objectptr thisobj, objinstptr thisinst)
 
 void freeselects()
 {
-   if (areastruct.selects > 0) {
-      if (areastruct.lastselects > 0)
-         free(areastruct.selectlast);
-      areastruct.selectlast = areastruct.selectlist;
-      areastruct.lastselects = areastruct.selects;
-      areastruct.selects = 0;
-   }
+   if (areastruct.selects > 0)
+      free(areastruct.selectlist);
+   areastruct.selects = 0;
 }
 
 /*--------------------------------------------------------------*/
@@ -850,7 +852,7 @@ void freeselects()
 /* items back to default values.				*/
 /*--------------------------------------------------------------*/
 
-void clearselects()
+void clearselects_noundo()
 {
    if (areastruct.selects > 0) {
       freeselects();
@@ -861,20 +863,16 @@ void clearselects()
 }
 
 /*--------------------------------------------------------------*/
-/* Free memory from both the current and previous selection	*/
-/* lists, because a page change has invalidated both.		*/
+/* Same as above, but registers an undo event.			*/
 /*--------------------------------------------------------------*/
 
-void destroyselects()
+void clearselects()
 {
-   if (areastruct.selects > 0) free(areastruct.selectlist);
-   if (areastruct.lastselects > 0) free(areastruct.selectlast);
-   areastruct.selects = 0;
-   areastruct.lastselects = 0;
- 
-   setallstylemarks(areastruct.style);
-   setcolormark(areastruct.color);
-   setdefaultfontmarks();
+   if (areastruct.selects > 0) {
+      register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
+		NULL, 0);
+      clearselects_noundo();
+   }
 }
 
 /*--------------------------------------------------------------*/
@@ -882,9 +880,9 @@ void destroyselects()
 /* selection list.						*/
 /*--------------------------------------------------------------*/
 
-void objectdeselect()
+void unselect_all()
 {
-   drawselects(topobject, areastruct.topinstance);
+   draw_normal_selected(topobject, areastruct.topinstance);
    clearselects();
 }
 
@@ -913,18 +911,18 @@ genericptr recurselect(short class, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   if ((selectobj = genobjectselect(class, selobj, selinst)) != NULL) {
+   if ((selectobj = genselectelement(class, UNDO_DONE, selobj, selinst)) != NULL) {
       rgen = *(selobj->plist + (*selectobj));
-      /* Fprintf(stdout, "%s found in object %s\n", (rgen->type == LABEL) ?
+      /* Fprintf(stdout, "%s found in object %s\n", (IS_LABEL(rgen)) ?
 			"label" : "polygon", selobj->name); */
-      drawselects(selobj, selinst);
+      draw_normal_selected(selobj, selinst);
       clearselects();
    }
-   else if ((selectobj = genobjectselect(OBJECT, selobj, selinst)) != NULL) {
+   else if ((selectobj = genselectelement(OBJINST, UNDO_DONE, selobj, selinst)) != NULL) {
       pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
 
       selinst = TOOBJINST(selobj->plist + (*selectobj));	
-      drawselects(selobj, selinst);
+      draw_normal_selected(selobj, selinst);
       clearselects();
 
       /* Link hierarchy information to the pushlist linked list */
