@@ -27,6 +27,13 @@ typedef unsigned long	u_long;
   #define Flush fflush
 #endif
 
+#ifdef TCL_WRAPPER
+  #define malloc Tcl_Alloc
+  #define calloc(a,b) Tcl_Alloc(a * b)
+  #define free(a) Tcl_Free((char *)(a))
+  #define realloc(a,b) Tcl_Realloc((char *)(a), b)
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Definition shortcuts 						*/
 /*----------------------------------------------------------------------*/
@@ -62,14 +69,25 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   (*(topobject->plist + *(a)))->type
+#define SELECTTYPE(a)   ((*(topobject->plist + *(a)))->type & ALL_TYPES)
 #define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
 #define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
 #define ENDPART		topobject->plist + topobject->parts
 #define EDITPART	topobject->plist + areastruct.editpart
 
-/* conversions from generic to specific types */
-/* specifically, from type (generic *) to type (polyptr), etc. */
+/* Type checks (argument "a" is type genericptr) */
+
+#define ELEMENTTYPE(a)	((a)->type & VALID_TYPES)
+
+#define IS_POLYGON(a)   (ELEMENTTYPE(a) == POLYGON)
+#define IS_LABEL(a)     (ELEMENTTYPE(a) == LABEL)
+#define IS_OBJINST(a)   (ELEMENTTYPE(a) == OBJINST)
+#define IS_ARC(a)       (ELEMENTTYPE(a) == ARC)
+#define IS_SPLINE(a)    (ELEMENTTYPE(a) == SPLINE)
+#define IS_PATH(a)      (ELEMENTTYPE(a) == PATH)
+
+/* Conversions from generic to specific types */
+/* specifically, from type (genericptr *) to type (polyptr), etc. */
 
 #define TOPOLY(a)	(*((polyptr *)(a)))
 #define TOLABEL(a)	(*((labelptr *)(a)))
@@ -104,7 +122,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	PLIST_INCR(b); \
 	a = (objinstptr *)b->plist + b->parts; \
 	*a = (objinstptr) malloc(sizeof(objinst)); \
-	(*a)->type = OBJECT
+	(*a)->type = OBJINST
 #define NEW_ARC(a,b) \
 	PLIST_INCR(b); \
 	a = (arcptr *)b->plist + b->parts; \
@@ -121,11 +139,21 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
 
-/* selection from top-level object */
+/*----------------------------------------------------------------------*/
+/* Undo mechanism definitions						*/
+/*----------------------------------------------------------------------*/
 
-#define objectselect(a)	 genobjectselect(a, topobject, areastruct.topinstance)
+enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE};
+
+/*----------------------------------------------------------------------*/
+/* selection from top-level object 					*/
+/*----------------------------------------------------------------------*/
+
+#define select_element(a) genselectelement(a, UNDO_MORE, topobject,\
+	areastruct.topinstance)
+#define select_add_element(a) genselectelement(a, UNDO_DONE, topobject,\
+	areastruct.topinstance)
 #define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
-#define drawselected(a)	 gendrawselected(a, topobject, areastruct.topinstance)
 
 /*----------------------------------------------------------------------*/
 
@@ -193,7 +221,7 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {NORMAL_MODE = 0, 
+enum editmode {NORMAL_MODE = 0, UNDO_MODE,
 PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
 PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
 CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
@@ -206,12 +234,12 @@ ULPARAM_MODE, UIPARAM_MODE, CONNECT_MODE, ASSOC_MODE};
 /* File loading modes							*/
 /*----------------------------------------------------------------------*/
 
-#define LOAD_MODES 5
-
-#define IMPORT     1
-#define PSBKGROUND 2
-#define SCRIPT	   3
-#define RECOVER    4
+enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
+#ifdef ASG
+	IMPORTSPICE,
+#endif
+	LOAD_MODES
+};
 
 /*----------------------------------------------------------------------*/
 /* Text justification styles and other parameters (bitmask)		*/
@@ -392,18 +420,27 @@ typedef XPoint* pointlist;
 typedef XfPoint* fpointlist;
 
 /*----------------------------------------------------------------------*/
-/* select types 							*/
+/* Basic element types 							*/
+/*									*/
+/* These values determine how a genericptr should be cast into one of	*/
+/* labelptr, polyptr, etc.  To query the element type, use the macros	*/
+/* ELEMENTTYPE(), IS_LABEL(), IS_POLYGON(), etc.  These macros mask the	*/
+/* bit field.  Note that an element marked with REMOVE_TAG will not be	*/
+/* recognized as any valid element, but otherwise, querying the type is	*/
+/* transparent to other bit settings (such as NETLIST_INVALID).		*/
 /*----------------------------------------------------------------------*/
 
-#define	OBJECT	1
+#define	OBJINST	1
 #define LABEL	2
 #define POLYGON	4
 #define ARC	8
 #define SPLINE  16
 #define PATH	32
-#define SEL_ANY	63  /* for selection mechanism only */
+#define ALL_TYPES 63
 
 #define REMOVE_TAG 64  /* add this value to objects to be removed */
+#define VALID_TYPES 127
+#define NETLIST_INVALID 128
 
 /*----------------------------------------------------------------------*/
 /* Allowed parameterization types					*/
@@ -1004,7 +1041,7 @@ enum {
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
    XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
-   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
+   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Move, XCF_Join, XCF_Unjoin, XCF_Spline,
    XCF_Edit, XCF_Undo, XCF_Redo, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
    XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
    XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
@@ -1102,6 +1139,7 @@ typedef struct _undostack {
    Undoptr next;	/* next record in undo stack */
    Undoptr last;	/* double-linked for "redo" function */
    u_int type;		/* type of event */
+   short idx;		/* counter for undo event */
    objinstptr thisinst;	/* instance of object in which event occurred */
    int idata;		/* simple undedicated integer datum */
    char *undodata;	/* free space to be malloc'd */
